// Copyright Erkki Seppl 2001-2002
// $Id: device.cc,v 1.8 2003/01/26 19:46:30 flux Exp $
// $Author: flux $ 
// $Revision: 1.8 $

static char* gs_id __attribute__ ((unused)) ="$Id: device.cc,v 1.8 2003/01/26 19:46:30 flux Exp $"; /// ID for CVS/RCS

#include "device.hh"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/wait.h>
#include <signal.h>

#include <iostream>

Device::Device(std::string a_name,
               std::string a_binary,
               Device* master) :
  m_name(a_name),
  m_binary(a_binary),
  m_child(0),
  m_master(master),
  m_recording(false)
{
  // nothing
}

Device::~Device()
{
  // nothing
}

void
Device::reinitialize(std::string a_binary,
                     Device* a_master)
{
  m_binary = a_binary;
  m_master = a_master;
}

std::string
Device::getName() const
{
  return m_name;
}

void
Device::cancel()
{
  if (m_child) {
    kill(m_child, SIGTERM);
    wait();
  }
}

bool
Device::available()
{
  check();
  return m_child == 0;
}

bool
Device::wait()
{
  if (m_child) {
    int status;
    if (waitpid(m_child, &status, 0) != 0) {
      m_child = 0;
      m_recording = false;
      return true;
    } else {
      return false;
    }
  } else {
    return false;
  }
}

void
Device::record(std::string a_channel,
               int         a_seconds,
               std::string a_file,
               const Arguments& a_arguments)
{
  if (m_child) {
    std::cout << "Device " << m_name << " is busy\n";
  } else {
    if ((m_child = fork()) == 0) {
      for (Arguments::const_iterator it = a_arguments.begin();
           it != a_arguments.end();
           ++it) {
        setenv(it->first.c_str(), it->second.c_str(), 1);
      }
      
      char buffer[1024];
      snprintf(buffer, sizeof(buffer), "%s %s %d %s",
               m_binary.c_str(), a_channel.c_str(), a_seconds, a_file.c_str());
      execl(getenv("SHELL"), getenv("SHELL"), "-c", buffer, 0);
      _exit(1);
    }
    m_recording = true;
  }
}

bool
Device::isRecording() const
{
  return m_recording;
}

void
Device::check()
{
  if (m_child) {
    int status;
    if (waitpid(m_child, &status, WNOHANG) != 0) {
      m_child = 0;
      m_recording = false;
    }
  }
}

// changelog:
//
// $Log: device.cc,v $
// Revision 1.8  2003/01/26 19:46:30  flux
// Committing most recent files to old repository backup
//
// Revision 1.8  2002/06/25 19:50:24  flux
// Added support for arguments
//
// Revision 1.7  2002/05/30 20:18:30  flux
// G++-3.0 -patches
//
// Revision 1.6  2002/05/17 18:58:33  flux
// Updated copyright message
//
// Revision 1.5  2001/11/15 14:32:23  flux
// Fixed typo
//
// Revision 1.4  2001/11/15 14:32:01  flux
// Added Device::reinitialize
//
// Revision 1.3  2001/11/15 14:27:41  flux
// Added Device::getName
//
// Revision 1.2  2001/11/14 20:03:20  flux
// Better detection of dead children
//
// Revision 1.1  2001/11/04 13:30:58  flux
// First version
//
