// Copyright Erkki Seppl 2001-2002
// $Id: device.hh,v 1.5 2003/01/26 19:46:30 flux Exp $
// $Author: flux $ 
// $Revision: 1.5 $


#ifndef DEVICE_HH
#define DEVICE_HH

#include <string>

#include "arguments.hh"

class Device {
public:
  Device(std::string a_name,
         std::string a_binary,
         Device* a_master);
  virtual ~Device();

  /// Reinitializes the device with new arguments
  void reinitialize(std::string a_binary,
                    Device* a_master);
  
  /// returns the name of the device
  std::string getName() const;
  
  /// returns true if this device is working and is not busy
  bool available();

  /// waits for the recording to stop
  bool wait();
  
  /// checks for dead children
  void check();

  /// cancel current recording
  void cancel();

  /// returns true if a show is being recorded
  bool isRecording() const;

  /// records given channel for given minutes to a given file
  virtual void record(std::string a_channel,
                      int 	  a_seconds,
                      std::string a_file,
                      const Arguments& a_arguments);
private:
  std::string 	m_name;
  std::string 	m_binary;
  int		m_child; /// 0 if no child
  Device*	m_master;
  bool		m_recording;
};

#endif // DEVICE_HH


// changelog:
//
// $Log: device.hh,v $
// Revision 1.5  2003/01/26 19:46:30  flux
// Committing most recent files to old repository backup
//
// Revision 1.5  2002/06/25 19:50:24  flux
// Added support for arguments
//
// Revision 1.4  2002/05/17 18:58:33  flux
// Updated copyright message
//
// Revision 1.3  2001/11/15 14:32:01  flux
// Added Device::reinitialize
//
// Revision 1.2  2001/11/15 14:27:41  flux
// Added Device::getName
//
// Revision 1.1  2001/11/04 13:30:58  flux
// First version
//
