#!/usr/bin/perl -w

# Copyright Erkki Seppl 2001-2002
# $Id: get-tv.pl,v 1.12 2003/01/26 19:46:30 flux Exp $
# $Author: flux $ 
# $Revision: 1.12 $

use Date::Calc;
use POSIX;

my ($showFile, $tvFile) = @ARGV;

if (!$showFile) {
  $showFile = "$ENV{HOME}/.mequ.shows"
}

if (!$tvFile) {
  $tvFile = "$ENV{HOME}/.mequ.tv";
}



local *F;

my $cmd = 'wget -qO- "$URL" |';

my %channels = ("TV1" => "http://tv.tupsu.ton.tut.fi/frames/single.php?channel_id=1", 
		"TV2" => "http://tv.tupsu.ton.tut.fi/frames/single.php?channel_id=2", 
		"MTV3" => "http://tv.tupsu.ton.tut.fi/frames/single.php?channel_id=3", 
		"Nelonen" => "http://tv.tupsu.ton.tut.fi/frames/single.php?channel_id=4", 
		"SUB" => "http://tv.tupsu.ton.tut.fi/frames/single.php?channel_id=5");

my @shows = ();

open(F, $showFile) or die "Cannot open $showFile";
while (<F>) {
  chomp;
  s/^#.*//;
  s/^ *//;
  s/ *$//;


  if ($_ ne "") {
    if (/^([0-9-]+)\s+\/(.+?)\/(?:\s*\{(.+)\})?\s+(.+)$/) {
      my $perlmatch = (defined $3) ? $3 : 1;
      my $show = { priority=>$1, match=>$2, perlmatch=>$perlmatch, name=>$4 };
      my @parts;
      ($show->{name}, @parts) = split(/ +/, $show->{name});
      %{$show->{env}} = map { my @p = split(/=/, $_); $p[0] => $p[1]; } @parts;
      push @shows, $show;
    } else {
      push @shows, { priority=>1, match=>$_, perlmatch=>"1", name=>$_, env=>{} };
    }
  }
}
close(F);

open(OUT, "> $tvFile-new") or die "Cannot open $tvFile";

my $success = 1;

my @time = localtime(time - 3.5 * 3600);
print OUT "# dynamically generated file by get-tv.pl from ~/.mequ.shows at " . ctime(time);
print OUT "# note that entries higher get better priority\n";

local %recorded;

foreach my $dayCounter (0..6) {

  local %list = ();

  foreach my $channel (sort keys %channels) {
    my $day = $time[6] + $dayCounter;
    
    $ENV{URL} = "$channels{$channel}&day=$dayCounter";
    
    #print "$ENV{URL}\n";

    local *F;
    open(F, $cmd);

    my $lastRecShow = "";
    my $lastRecTime = "";
    my $lastRecDay = "";
    my $lastTime = "";

    while (<F>) {
      # last show wont be recorded!
      if (my ($time, $show) = /^<tr.*?<b>([^&]+).*?<b>(.+).*?<\/b>/) {
	#print "$time: $show\n";
	if ($lastRecTime ne "") {
	  add($lastRecDay, $lastRecTime, $channel, $time, $lastRecPriority, 
              $lastRecShow, $lastInfo);
	}
	$lastRecTime = "";

	if ($lastTime ne "" && $time lt $lastTime) {
	  ++$day;
	}
	$lastTime = $time;

	for (my $c = 0; $c < @shows; ++$c) {
	  if ($show =~ /$shows[$c]->{match}/i) {
            $lastInfo = { fullname=>$show, perlmatch=>$shows[$c]->{perlmatch} };
	    $lastRecShow = $shows[$c];
	    $lastRecTime = $time;
	    $lastRecDay = $day;
	    $lastRecPriority = $c;
	  }
	}
      }
    }

    if ($lastRecTime ne "") {
      print "$lastRecShow->{name} ending time not known! Guessing 60 minutes\n";
      $time = "60m";
      add($lastRecDay, $lastRecTime, $channel, $time, $lastRecPriority, 
          $lastRecShow, $lastInfo);
    }

    $success = close(F);
  }

  foreach my $priority (sort { $a <=> $b } keys %list) {
    print OUT join("", @{$list{$priority}});
  }

  if (!$success) {
    last;
  }
}

close(OUT);

if ($success) {
  my @notFound;
  for (my $c = 0; $c < @shows; ++$c) {
    if (!exists $recorded{$shows[$c]->{name}}) {
      push @notFound, $shows[$c]->{name};
    }
  }
  if (@notFound) {
    print "Could not find:\n\t", join("\n\t", sort @notFound), "\n";
  }
  rename $tvFile, "$tvFile~";
  rename "$tvFile-new", $tvFile;
  exit(0);
} else {
  print "Retrieving shows failed";
  exit(1);
}

sub add {
  my ($recDay, $recTime, $channel, $time, $sortPriority, $recShow, $recInfo) = @_;
  $recorded{$recShow->{name}} = 1; # actually used for showing the handled shows at the end
  my ($extra, $fullname, $day, $begin, $end, $name, $priority,) = 
    (15, $recInfo->{fullname}, $recDay, $recTime, $time, $recShow->{name}, $recShow->{priority});
  my %env = %{$recShow->{env}};
  if (eval $recInfo->{perlmatch}) {
    $name =~ s/ /_/g;
    push @{$list{$sortPriority}}, $recDay % 7, 
      "/$begin	$end+$extra	h1	$channel	$priority	$name" . 
	join("", map { " $_=$env{$_}" } keys %env) . "\n";
  }
}

# changelog:
#
# $Log: get-tv.pl,v $
# Revision 1.12  2003/01/26 19:46:30  flux
# Committing most recent files to old repository backup
#
# Revision 1.12  2002/06/25 19:25:12  flux
# Fixed a name printin bug
#
# Revision 1.11  2002/05/30 20:39:22  flux
# Added support for more advanced show listing format
#
# Revision 1.10  2002/05/26 20:49:53  flux
# Now makes a backup of the original
#
# Revision 1.9  2002/05/25 17:48:24  flux
# Changed still 9 minutes to 15 minutes
#
# Revision 1.8  2002/05/20 19:10:46  flux
# Now it tells if it couldn't find some shows.
# 9 minutes spare instead of 5.
#
# Revision 1.7  2002/05/17 20:51:07  flux
# Added CVS-keywords
#
