// Copyright Erkki Seppl 2001-2002
// $Id: mediaqueue.hh,v 1.6 2002/05/17 18:58:33 flux Exp $
// $Author: flux $ 
// $Revision: 1.6 $

#ifndef MEDIAQUEUE_HH
#define MEDIAQUEUE_HH

#include <string>
#include <memory>
#include <list>
#include <map>

class Show;

typedef std::list<Show*> ShowList;
typedef std::map<time_t, Show*> TimeShowMap;

class Device;
typedef std::list<Device*> DeviceList;

class MediaQueue {
public:
  typedef Show* value_type;
  typedef ShowList::iterator iterator;
  typedef ShowList::const_iterator const_iterator;
  
  MediaQueue(DeviceList& a_devices);
  virtual ~MediaQueue();

  /// Finds a given device name
  Device* findDevice(std::string deviceName) const;
  
  void addShow(Show* show);
  void schedule(time_t a_begin,
                time_t a_end,
                TimeShowMap& a_shown,
                TimeShowMap& a_skip);
  
  iterator begin();
  iterator end();
  const_iterator begin() const;
  const_iterator end() const;

private:
  MediaQueue(const MediaQueue& a_other); /// not there
  MediaQueue& operator=(const MediaQueue& a_other); /// not there

  ShowList 	m_shows;
  DeviceList&	m_devices;
};

void schedule(ShowList& a_shows,
              time_t a_begin,
              time_t a_end,
              TimeShowMap& a_shown,
              TimeShowMap& a_skip);

void showSchedule(std::ostream& a_stream, const TimeShowMap& a_times);
std::ostream& operator<<(std::ostream& a_stream,
                         const MediaQueue& a_queue);

#endif // MEDIAQUEUE_HH


// changelog:
//
// $Log: mediaqueue.hh,v $
// Revision 1.6  2002/05/17 18:58:33  flux
// Updated copyright message
//
// Revision 1.5  2001/11/15 15:31:56  flux
// Add stream-argument to showSchedule
//
// Revision 1.4  2001/11/15 15:10:35  flux
// Moved loading configuration out of MediaQueue
//
// Revision 1.3  2001/11/15 14:54:08  flux
// Added Device::findDevice
//
// Revision 1.2  2001/11/15 14:34:22  flux
// Added MediaQueue::transferQueue to transfer existing devices from old MediaQueue instances.
// Added an extra argument MediaQueue* old to MediaQueue::load for the same reason.
//
// Revision 1.1  2001/11/04 13:30:58  flux
// First version
//
