// Copyright Erkki Seppl 2001-2002
// $Id: mequ.cc,v 1.45 2003/02/16 02:02:00 flux Exp $
// $Author: flux $ 
// $Revision: 1.45 $

static char* gs_id __attribute__ ((unused)) ="$Id: mequ.cc,v 1.45 2003/02/16 02:02:00 flux Exp $"; /// ID for CVS/RCS

#include <unistd.h>
#include <signal.h>

#include <string>
#include <fstream>
#include <memory>
#include <vector>
#include <algorithm>
#include <map>

#include <string.h>

#include "mediaqueue.hh"
#include "device.hh"
#include "occurrence.hh"
#include "show.hh"
#include "split.hh"
#include "server.hh"

bool requestReload = false;

void huphandler(int sig)
{
  signal(sig, huphandler);
  requestReload = true;
}

void
help()
{
  std::cout << "usage: mequ [--show|--daemon|--stop|--reload|--days n|--hours n|--rows n] [mequ config file]" << std::endl;
}

template <class T>
class StringOp {
public:
  typedef const char* first_argument_type;
  typedef const char* second_argument_type;
  typedef bool result_type;
  StringOp(T t) : t(t){}
  bool operator()(const char* s1, const char* s2) const { return !(*t)(s1, s2); }
private:
  T t;
};

template <class T>
StringOp<T>
stringOp(T t)
{
  return StringOp<T>(t);
}

typedef std::map<std::string, Device*> NameDeviceMap;

void
loadConfiguration(std::string 	a_filename,
                  MediaQueue* 	a_mequ,
                  DeviceList* 	a_devices,
                  std::ostream** a_log,
                  std::string* 	a_pidfile,
                  std::string* 	a_socketfile,
                  int* 		a_row = 0,
                  NameDeviceMap* a_deviceMap = 0)
{
  using namespace std;

  NameDeviceMap devices;

  if (!a_deviceMap) {
    a_deviceMap = &devices;
  }

  int row = 0;
  if (!a_row) {
    a_row = &row;
  }
  
  time_t now = time(0);
  struct tm nowTm = *localtime(&now);
  time_t weekTime = (now
                     - nowTm.tm_sec
                     - nowTm.tm_min * 60
                     - nowTm.tm_hour * 60 * 60
                     - nowTm.tm_wday * 24 * 60 * 60);
  ifstream file(a_filename.c_str());
  if (!file) {
    std::cerr << "Failed to open file " << a_filename << "\n";
    return;
  }
  std::string when;
  while (file) {
    ++*a_row;
    string line;
    if (getline(file, line)) {
      std::string::size_type offset = line.find('#');
      if (offset != std::string::npos) {
        line = line.substr(0, offset);
      }

      vector<string> cmd;
      split(line, " \t", back_inserter(cmd), false);
      if (cmd.size() == 2 &&
          cmd[0] == "log") {
        if (a_log) {
          delete *a_log;
          *a_log = new std::ofstream(cmd[1].c_str());
        }
      } else if (cmd.size() >= 1 &&
                 cmd[0] == "echo") {
        std::copy(cmd.begin() + 1, cmd.end(), std::ostream_iterator<std::string>(std::cout, "#"));
        std::cout << "\n";
      } else if (cmd.size() == 2 &&
                 cmd[0] == "pidfile") {
        if (a_pidfile) {
          *a_pidfile = cmd[1];
        }
      } else if (cmd.size() == 2 &&
                 cmd[0] == "socket") {
        if (a_socketfile) {
          *a_socketfile = cmd[1];
        }
      } else if (cmd.size() == 5 &&
                 cmd[0] == "device") {
        if (a_devices) {
          Device* device = 0;
          NameDeviceMap::iterator deviceIt = a_deviceMap->find(cmd[2]);
          if (deviceIt != a_deviceMap->end()) {
            device = deviceIt->second;
            device->reinitialize(cmd[4], (*a_deviceMap)[cmd[3]]);
          } else {
            device = new Device(cmd[2], cmd[4], (*a_deviceMap)[cmd[3]]);
            a_devices->push_back(device);
          }
          (*a_deviceMap)[cmd[1]] = device;
        }
      } else if (cmd.size() == 2 &&
                 cmd[0] == "include") {
        loadConfiguration(cmd[1], a_mequ, a_devices, a_log, a_pidfile, a_socketfile, a_row, a_deviceMap);
      } else if (cmd.size() >= 6) {
        Arguments arguments;
        for (unsigned c = 6; c < cmd.size(); ++c) {
          std::string& argValue = cmd[c];
          bool handled = false;
          for (unsigned d = 0; d < argValue.size() && !handled; ++d) {
            if (argValue[d] == '=') {
              arguments[argValue.substr(0, d)] = argValue.substr(d + 1);
              handled = true;
            }
          }
          if (!handled) {
            std::cerr << "Warning: argument " << argValue << " is not valid\n";
          }
        }
        if (a_mequ && a_devices) {
          vector<string> time;
          if (splitSeq(cmd[0], "/:", back_inserter(time)) == 3) {
            if (time[0] != "") {
              when = time[0];
            }
            int duration = 0;
            int subDuration = 0;
	    int extraTime = -1;
            for (unsigned c = 0; c < cmd[1].size(); ++c) {
              if (cmd[1][c] >= '0' && cmd[1][c] <= '9') {
                subDuration = subDuration * 10 + (cmd[1][c] - '0');
              } else switch (cmd[1][c]) {
              case 'm': {
                duration += subDuration;
                subDuration = 0;
              } break;
              case 'h': {
                duration += subDuration * 60;
                subDuration = 0;
              } break;
              case ':': {
                duration = (subDuration - atoi(time[1].c_str())) * 60 - atoi(time[2].c_str());
                subDuration = 0;
              } break;
	      case '+': {
		duration += subDuration;
		subDuration = 0;
		extraTime = 1;
	      } break;
              }
            }

	    if (extraTime == -1) {
	      duration += subDuration;
              extraTime = 0;
	    } else {
	      extraTime = subDuration;
	    }

            while (duration < 0) {
              duration += 24 * 60;
            }

            int priority = (atoi(cmd[4].c_str()) * (1 << 16)) + (*a_row & 0xffff);
          
            if (when.size() >= 8) {
              struct tm exactTm;
              memset(&exactTm, 0, sizeof(exactTm));
              exactTm.tm_mday = atoi(when.substr(6, 2).c_str());
              exactTm.tm_mon = atoi(when.substr(4, 2).c_str()) - 1;
              exactTm.tm_year = atoi(when.substr(0, 4).c_str()) - 1900;
              exactTm.tm_hour = atoi(time[1].c_str());
              exactTm.tm_min = atoi(time[2].c_str());
              exactTm.tm_isdst = -1;
              a_mequ->addShow(new Show(cmd[5], duration, priority,
                                       (*a_deviceMap)[cmd[2]], cmd[3],
                                       new OneShotOccurrence(mktime(&exactTm)),
				       extraTime, arguments));
            } else {
              if (when.size() > 0 && when[when.size() - 1] == '/') {
                when = when.substr(0, when.size() - 1);
              }
              int dayNum = atoi(when.c_str()) % 7;

              static const char* weekdays[2][7] = { { "sun", "mon", "tue", "wed", "thu", "fri", "sat" },
                                                    { "su", "ma", "ti", "ke", "to", "pe", "la" } };
              for (int language = 0; language < 2; ++language) {
                const char** weekday = std::find_if(weekdays[language], weekdays[language] + 7,
                                                    std::bind1st(stringOp(strcasecmp), when.c_str()));
                // if it was a valid weekday, use it instead
                if (weekday != weekdays[language] + 7) {
                  dayNum = weekday - weekdays[language];
                  break;
                }
              }
              a_mequ->addShow(new Show(cmd[5], duration, priority,
                                       (*a_deviceMap)[cmd[2]], cmd[3],
                                       new CycleOccurrence(weekTime +
                                                           24 * 60 * 60 * dayNum +
                                                           atoi(time[1].c_str()) * 60 * 60 +
                                                           atoi(time[2].c_str()) * 60,
                                                           7 * 24 * 60 * 60),
				       extraTime, arguments));
            }
#if 0
            Show(std::string 	a_name,
                 int 		a_minutes,
                 int		a_priority,
                 Device* 		a_device,
                 std::string	a_channel,
                 Occurrence*	a_occurrence = 0);
#endif

          } else {
            std::cerr << "Invalid input" << std::endl;
          }
        }
      } else if (cmd.size() == 1) {
        when = cmd[0];
      }
    }
  }
}

void
showSchedule(std::ostream& a_stream,
             int a_maxRows,
             const TimeShowMap& a_show,
             const TimeShowMap& a_dontShow)
{
  TimeShowMap::const_iterator showIt = a_show.begin();
  TimeShowMap::const_iterator dontIt = a_dontShow.begin();
  int totalShow = 0;
  int totalDont = 0;
  if (!a_maxRows) {
    a_maxRows = a_show.size() + a_dontShow.size();
  }
  time_t now = time(0);
  while ((showIt != a_show.end() ||
          dontIt != a_dontShow.end()) &&
         a_maxRows > 0) {
    enum { NONE, SHOW, DONT } show = NONE;
    if (showIt == a_show.end()) {
      show = DONT;
    } else if (dontIt == a_dontShow.end()) {
      show = SHOW;
    } else if (showIt->first < dontIt->first) {
      show = SHOW;
    } else {
      show = DONT;
    }
    switch (show) {
    case SHOW: {
      a_stream << " " << formatTime(showIt->first, "%a %Y-%m-%d %H:%M")
               << (overlaps(showIt->first, showIt->second->getMinutes(), now, 0) ? 
                   " R " : " | ")
               << *showIt->second << std::endl;
      totalShow += showIt->second->getMinutes();
      ++showIt;
    } break;
    case DONT: {
      a_stream << "(" << formatTime(dontIt->first, "%a %Y-%m-%d %H:%M") << " - " << *dontIt->second << ")" << std::endl;
      totalDont += dontIt->second->getMinutes();
      ++dontIt;
    } break;
    case NONE: break;
    }
    --a_maxRows;
  }
  a_stream << "record total " << totalShow / 60 << "h" << totalShow % 60 << "m, ";
  a_stream << "skip total " << totalDont / 60 << "h" << totalDont % 60 << "m\n";
}

void
show(int a_seconds, int a_maxRows, std::string a_configFile)
{
  DeviceList devices;
  std::auto_ptr<MediaQueue> queue(new MediaQueue(devices));
  loadConfiguration(a_configFile, queue.get(), &devices, 0, 0, 0);
  time_t now = time(0);
  TimeShowMap shown, skipped;
  queue->schedule(now, now + a_seconds, shown, skipped);

  showSchedule(std::cout, a_maxRows, shown, skipped);

  for (DeviceList::iterator it = devices.begin();
       it != devices.end();
       ++it) {
    delete *it;
  }
}

void
daemon(std::string a_configFile, bool foreground)
{
  std::string pidfile = "";
  std::string socketfile = "";
  std::ostream* log =
    foreground ? &std::cout : new std::ofstream("/dev/null");
  DeviceList devices;
  std::auto_ptr<MediaQueue> queue(new MediaQueue(devices));
  std::map<Device*, Show*> lastRecording;
  
  loadConfiguration(a_configFile, queue.get(), &devices,
                    foreground ? 0 : &log, &pidfile, &socketfile);

  if (pidfile == "") {
    std::cerr << "No pid file\n";
    return;
  }
  
  if (socketfile == "") {
    std::cerr << "No socket file\n";
    return;
  }
  
  (*log) << "shows: \n" << *queue;
  int child = foreground ? 0 : fork();
  if (child == 0) { // child process
    signal(SIGHUP, huphandler);
    Server server(socketfile, *log);
    if (!server) {
      (*log) << "Cannot start server-portion" << std::endl;
      return;
    }
    while (true) {
      time_t now = time(0);
      TimeShowMap shown, skipped;
      queue->schedule(now + 30, now + 24 * 60 * 60, shown, skipped);
      (*log) << "shown: " << std::endl;
      showSchedule(*log, shown);
      (*log) << "skipped: " << std::endl;
      showSchedule(*log, skipped);
      if (shown.size()) {
        TimeShowMap::iterator programIt;
        Device* device;
        Show* show;
        time_t showTime;
        do {
          if (shown.size()) {
            programIt = shown.begin();
            show = programIt->second;
            device = programIt->second->getDevice();
            showTime = programIt->first;
            shown.erase(programIt);
          } else {
            show = 0;
            break;
          }
          if (device) {
            device->check();
          }
        } while (device && show &&
                 ((device->isRecording() && lastRecording[device] == show) ||
                  show->getLastRecording() == showTime));
        if (show) {
          if (device) {
            (*log) << "To record: " << show->getName() << " at "
                      << formatTime(showTime, "%c") << std::endl;
            int ok = true;
            if (showTime >= now) {
              (*log) << "Sleeping for " << showTime - now << " seconds" << std::endl;
              ok = server.sleepAndServe(showTime - now) == 0;
            }
            device->check();
            if (ok) {
              if (device->available()) {
                now = time(0);
                time_t end = showTime + (show->getMinutes() + show->getExtra()) * 60;
                if (shown.size()) {
                  if (end > shown.begin()->first) {
                    end = shown.begin()->first;
                  }
                }
                int duration = end - now;
                (*log) << "Recording for " << duration / 60 << " minutes" << std::endl;
                // get some spare for deinitialization of the device
                if (duration > 10 &&
                    (duration > show->getMinutes() * 60 / 10 ||
                     duration > 10 * 60)) {
                  device->record(show->getChannel(), duration - 10, show->getName(), show->getArguments());
                  lastRecording[device] = show;
                  show->setLastRecording(showTime);
                } else {
                  (*log) << "Skipped recording of " << duration << " seconds" << std::endl;
                }
              } else {
                (*log) << "Device is not available!" << std::endl;
              }
            }
          } else {
            (*log) << "No device!" << std::endl;
          }
          sleep(1);
          if (requestReload) {
            requestReload = false;
            std::auto_ptr<MediaQueue> newQueue(new MediaQueue(devices));
            std::swap(queue, newQueue);
            loadConfiguration(a_configFile, queue.get(), &devices, &log, &socketfile, 0);
            now = time(0);
            queue->schedule(now, now + 24 * 60 * 60, shown, skipped);
            device->check();
            if (!device->available()) {
              if (shown.size()) {
                if (!device->isRecording() ||
                    lastRecording[device] != shown.begin()->second) {
                  device->cancel();
                } else {
                  device->wait();
                }
              } else {
                device->wait();
              }
            }
          }
        } else {
          (*log) << "No show: sleeping for 12 hours" << std::endl;
          server.sleepAndServe(12 * 3600);
        }
      }
      sleep(1);
    }
    for (DeviceList::iterator it = devices.begin();
         it != devices.end();
         ++it) {
      delete *it;
    }
    if (log != &std::cout) {
      delete log;
    }
    _exit(0);
  } else {
    if (pidfile != "") {
      std::ofstream stream(pidfile.c_str());
      stream << child << std::endl;
    } else {
      std::cout << child << std::endl;
    }
    for (DeviceList::iterator it = devices.begin();
         it != devices.end();
         ++it) {
      delete *it;
    }
    delete log;
  }
}

void
reload(std::string a_configFile)
{
  std::string pidfile;
  loadConfiguration(a_configFile, 0, 0, 0, &pidfile, 0);

  std::ifstream stream(pidfile.c_str());
  int pid;
  stream >> pid;

  if (stream) {
    if (!pid) {
      std::cout << "No MEQU around!\n";
    } else {
      kill(pid, SIGHUP);
    }
  } else {
    std::cout << "Failed to read pidfile\n";
  }
}

void
stop(std::string a_configFile)
{
  std::string pidfile;
  loadConfiguration(a_configFile, 0, 0, 0, &pidfile, 0);

  std::ifstream stream(pidfile.c_str());
  int pid;
  stream >> pid;

  if (stream) {
    if (!pid) {
      std::cout << "No MEQU around!\n";
    } else {
      kill(pid, SIGTERM);
    }
  } else {
    std::cout << "Failed to read pidfile\n";
  }
}

int
main(int argc, char** argv)
{
  std::string configFile;
  int argOffset = 1;
  bool foreground = false;
  int showSeconds = 0;
  int showMaxRows = 0;

  enum { MODE_SHOW, MODE_DAEMON, MODE_RELOAD, MODE_STOP } mode = MODE_SHOW;
  
  for (int c = 1; c < argc; ++c) {
    if (strcmp("--show", argv[c]) == 0) {
      mode = MODE_SHOW;
      argOffset = c + 1;
    } else if (strcmp("--daemon", argv[c]) == 0) {
      mode = MODE_DAEMON;
      argOffset = c + 1;
    } else if (strcmp("--foreground", argv[c]) == 0) {
      foreground = true;
      argOffset = c + 1;
    } else if (strcmp("--reload", argv[c]) == 0) {
      mode = MODE_RELOAD;
      argOffset = c + 1;
    } else if (strcmp("--stop", argv[c]) == 0) {
      mode = MODE_STOP;
      argOffset = c + 1;
    } else if (strcmp("--days", argv[c]) == 0) {
      showSeconds += atoi(argv[c + 1]) * 24 * 3600;

      c++;
      argOffset = c + 1;
    } else if (strcmp("--hours", argv[c]) == 0) {
      showSeconds += atoi(argv[c + 1]) * 3600;

      c++;
      argOffset = c + 1;
    } else if (strcmp("--rows", argv[c]) == 0 ||
               strcmp("--maxrows", argv[c]) == 0) {
      showMaxRows = atoi(argv[c + 1]);

      c++;
      argOffset = c + 1;
    } else {
      if (strncmp("--", argv[c], 2) == 0) {
        help();
        return 0;
      } else {
        break;
      }
    }
  }
  
  if (argOffset < argc) {
    configFile = argv[argOffset];
    ++argOffset;
  } else {
    configFile = std::string(getenv("HOME")) + "/.mequ";
  }

  switch (mode) {
  case MODE_SHOW: {
    if (showSeconds == 0) {
      showSeconds = 7 * 24 * 3600;
    }
    show(showSeconds, showMaxRows, configFile);
  } break;
  case MODE_DAEMON: {
    daemon(configFile, foreground);
  } break;
  case MODE_RELOAD: {
    reload(configFile);
  } break;
  case MODE_STOP: {
    stop(configFile);
  } break;
  }
  
  return 0;
}

// changelog:
//
// $Log: mequ.cc,v $
// Revision 1.45  2003/02/16 02:02:00  flux
// Better handling of --foreground (disable logging to file)
//
// Revision 1.44  2003/01/27 07:19:15  flux
// Added --rows in preference to --maxrows
//
// Revision 1.43  2003/01/26 19:46:30  flux
// Committing most recent files to old repository backup
//
// Revision 1.43  2002/06/25 19:51:04  flux
// Added support for arguments.
// Added more error reporting.
//
// Revision 1.42  2002/06/09 22:19:52  flux
// Added capability to show only given number of rows (--maxrows)
//
// Revision 1.41  2002/06/09 20:35:52  flux
// Better showing of schedule (interleaved shown/skipped)
//
// Revision 1.40  2002/05/30 20:18:30  flux
// G++-3.0 -patches
//
// Revision 1.39  2002/05/27 20:53:28  flux
// Added --days and --hours to help text
//
// Revision 1.38  2002/05/27 18:39:47  flux
// Reverted old behavior: show a week (by default) instead of one day
//
// Revision 1.37  2002/05/26 20:50:10  flux
// Added --days and --hours to show only a certain period
//
// Revision 1.36  2002/05/17 18:58:33  flux
// Updated copyright message
//
// Revision 1.35  2002/05/15 17:33:20  flux
// Removed 30 second delay from showing the schedule
//
// Revision 1.34  2002/05/14 14:33:40  flux
// Actually adjust the show if it overlaps
//
// Revision 1.33  2002/05/14 12:30:56  flux
// Actually construct the show with the extra minutes
//
// Revision 1.32  2002/05/14 08:18:50  flux
// Fixed include-file support (devices not found)
//
// Revision 1.31  2002/05/13 20:39:20  flux
// Row number now spans over different configuration files (priorities work correctly)
//
// Revision 1.30  2002/05/13 19:06:30  flux
// Added --foreground for debugging
//
// Revision 1.29  2002/05/12 20:28:23  flux
// Added support for extra minutes
//
// Revision 1.28  2002/05/12 19:44:53  flux
// Reindented.
// Added 'include'.
//
// Revision 1.27  2002/04/21 20:24:43  flux
// Preliminary support for "extra minutes".
// Fixed daylight saving time-related bug
//
// Revision 1.26  2001/11/16 20:46:39  flux
// Updated help-screen
//
// Revision 1.25  2001/11/16 20:44:17  flux
// Moved splitting out of mequ.cc
//
// Revision 1.24  2001/11/16 20:42:12  flux
// Modified the output.
//
// Revision 1.23  2001/11/16 20:40:23  flux
// Added support for using day names instead of numbers in configuration file. Both in English and Finnish.
//
// Revision 1.22  2001/11/16 20:11:34  flux
// Removed templatization from MatchSequence
//
// Revision 1.21  2001/11/16 20:02:36  flux
// Use templates instead of a parameter to MatchQuote
//
// Revision 1.20  2001/11/16 20:01:23  flux
// Added capability to MatchQuote to choose whether to use submatcher before or after decision.
//
// Revision 1.19  2001/11/16 19:56:19  flux
// Added MatchQuote, and changed split by character set to use that
//
// Revision 1.18  2001/11/16 19:20:03  flux
// Added MatchAny to match a range of characters.
// Can now bypass whitespaces.
// Handles comments better.
//
// Revision 1.17  2001/11/16 13:59:56  flux
// Duration of the show can now be entered as when the show ends
//
// Revision 1.16  2001/11/16 13:46:36  flux
// Now doesn't consume all CPU if there are no shows to show.
// Notifies the user when it skips a recording.
//
// Revision 1.15  2001/11/15 16:03:30  flux
// Added --stop
//
// Revision 1.14  2001/11/15 16:00:22  flux
// Added reloading.
// Added pidfile-support.
// Fixed problem in handling devices during reload.
//
// Revision 1.13  2001/11/15 15:19:02  flux
// Better support for remembering old devices
//
// Revision 1.12  2001/11/15 15:14:06  flux
// Moved show and daemon to separate functions
//
// Revision 1.11  2001/11/15 15:10:35  flux
// Moved loading configuration out of MediaQueue
//
// Revision 1.10  2001/11/15 14:34:50  flux
// Fill the MediaQueue* old-parameter of MediaQueue::load
//
// Revision 1.9  2001/11/15 14:21:14  flux
// Added daemonization.
// Show schedule by default.
// Handle SIGHUP instead of SIGUSR1.
//
// Revision 1.8  2001/11/15 08:42:54  flux
// Added command line help
//
// Revision 1.7  2001/11/15 08:41:15  flux
// Stop processing command line arguments if an invalid switch is found.
//
// Revision 1.6  2001/11/15 08:40:45  flux
// Now skips recorded shows better.
//
// Revision 1.5  2001/11/14 21:03:31  flux
// Added some support for storing the time of last recording in Show
//
// Revision 1.4  2001/11/14 20:21:12  flux
// Added --show -switch
//
// Revision 1.3  2001/11/14 20:04:13  flux
// Added one more device check point.
// Cut 10 seconds out of record time.
// Record only if the recorded time is 1/10 of the total time or it's longer than 10 minutes.
//
// Revision 1.2  2001/11/13 19:53:58  flux
// If the show will stop in next 30 seconds, don't start recording it.
// Gets the configuration file from command line
//
// Revision 1.1  2001/11/04 13:30:58  flux
// First version
//
