// Copyright Erkki Seppl 2002
// $Id: address.cc,v 1.7 2003/01/26 19:40:39 flux Exp $
// $Author: flux $ 
// $Revision: 1.7 $

static char* gs_id __attribute__ ((unused)) ="$Id: address.cc,v 1.7 2003/01/26 19:40:39 flux Exp $"; /// ID for CVS/RCS

#include <network/address.hh>

#include <cstdio>
#include <cstdlib>

namespace Network {
  Address::Interface* Address::s_interfaces[256] = { 0 };

  Address::Address() : m_address(0)
  {
    // nothing
  }

  Address::Address(const Address& a_other)
  {
    int size;
    const sockaddr* addr = a_other.get(&size);
    if (addr) {
      m_address = (sockaddr*) std::malloc(size);
      std::memcpy(m_address, addr, size);
    } else {
      m_address = 0;
    }
  }

  Address::Address(const void* a_address)
  {
    sockaddr* addr = (sockaddr*) a_address;
    
    m_address = 0;
    
    if (a_address) {
      int size = getInterface(addr)->getSize(addr);
      m_address = (sockaddr*) malloc(size);
      std::memcpy(m_address, a_address, size);
    }
  }

  Address&
  Address::operator=(const Address& a_other)
  {
    if (&a_other != this) {
      if (m_address) {
        free(m_address);
      }

      int size;
      const sockaddr* addr = a_other.get(&size);
      if (addr) {
        m_address = (sockaddr*) malloc(size);
        std::memcpy(m_address, addr, size);
      } else {
        m_address = 0;
      }
    }
    return *this;
  }

  
  Address::~Address()
  {
    if (m_address) {
      free(m_address);
    }
  }

  const sockaddr*
  Address::get(int* a_size) const
  {
    if (a_size) {
      *a_size = getInterface(m_address)->getSize(m_address);
      return m_address;
    } else {
      return m_address;
    }
  }

  sockaddr*
  Address::get(int* a_size)
  {
    if (a_size) {
      *a_size = getInterface(m_address)->getSize(m_address);
      return m_address;
    } else {
      return m_address;
    }
  }

  const sockaddr*
  Address::getAddress() const
  {
    return m_address;
  }

  socklen_t
  Address::getSize() const
  {
    return getInterface(m_address)->getSize(m_address);
  }

  void
  Address::set(const void* a_address)
  {
    if (m_address) {
      free(m_address);
    }
    
    if (a_address) {
      int size = getInterface((sockaddr*) a_address)->getSize((sockaddr*) a_address);
      m_address = (sockaddr*) malloc(size);
      std::memcpy(m_address, a_address, size);
    } else {
      m_address = 0;
    }
  }

  std::string
  Address::asString() const
  {
    return getInterface(m_address)->asString(m_address);
  }
}

std::ostream&
operator <<(std::ostream& a_stream, const Network::Address& address)
{
  a_stream << address.asString();
  return a_stream;
}


// changelog:
//
// $Log: address.cc,v $
// Revision 1.7  2003/01/26 19:40:39  flux
// Added some files (had to revert an old repository from backups)
//
// Revision 1.7  2002/07/02 19:01:15  flux
// Added non-const protected version of get.
// Added 'explicit' to void*-constructor to avoid problems.
//
// Revision 1.6  2002/05/28 14:55:04  flux
// Updated copyright message
//
// Revision 1.5  2002/05/28 09:58:18  flux
// Fixed a bug in .get when it received a zero argument
//
// Revision 1.4  2002/05/22 08:33:29  flux
// Added some Socket-wrapping
//
// Revision 1.3  2002/05/22 07:13:16  flux
// Added another interface to Address
//
// Revision 1.2  2002/05/22 07:03:04  flux
// Added error-handling.
// Added casting.
// Added type checking method.
//
// Revision 1.1  2002/05/22 06:33:43  flux
// First version
//
