// Copyright Erkki Seppl 2002
// $Id: address.hh,v 1.6 2003/01/26 19:40:39 flux Exp $
// $Author: flux $ 
// $Revision: 1.6 $


#ifndef ADDRESS_HH
#define ADDRESS_HH

#include <string>
#include <iostream>

#include <sys/socket.h>
#include <sys/types.h>
#include <network/error.hh>

namespace Network {
  class Address {
  public:
    Address();

    // makes a copy
    explicit Address(const void* a_address);
    Address(const Address& a_other);
    Address& operator=(const Address& a_other);
    ~Address();
    
    void set(const void* a_address);
    socklen_t getSize() const;
    const sockaddr* getAddress() const;
    const sockaddr* get(int* size) const;
    
    std::string asString() const;

    template <class T> T& cast() {
      if (T::getProtocol() == m_address->sa_family) {
        return static_cast<T&>(*this);
      } else {
        error(ERROR_FAMILY, "Wrong protocol family in cast");
      }
    }

    template <class T> bool is() {
      return (T::getProtocol() == m_address->sa_family);
    }

  protected:
    sockaddr* get(int* size);

    class Interface {
    public:
      virtual std::string asString(const sockaddr* a_address) const = 0;
      virtual socklen_t getSize(const sockaddr* a_address) const = 0;
    };
    
    static Interface* s_interfaces[256];

    inline static Interface* getInterface(sockaddr* a_address) {
      return s_interfaces[a_address->sa_family];
    }

  private:

    
    sockaddr*	m_address;
  };
}

std::ostream& operator <<(std::ostream& a_stream, const Network::Address& address);

#endif // ADDRESS_HH


// changelog:
//
// $Log: address.hh,v $
// Revision 1.6  2003/01/26 19:40:39  flux
// Added some files (had to revert an old repository from backups)
//
// Revision 1.6  2002/07/02 19:01:15  flux
// Added non-const protected version of get.
// Added 'explicit' to void*-constructor to avoid problems.
//
// Revision 1.5  2002/05/28 14:55:04  flux
// Updated copyright message
//
// Revision 1.4  2002/05/22 08:33:29  flux
// Added some Socket-wrapping
//
// Revision 1.3  2002/05/22 07:13:16  flux
// Added another interface to Address
//
// Revision 1.2  2002/05/22 07:03:04  flux
// Added error-handling.
// Added casting.
// Added type checking method.
//
// Revision 1.1  2002/05/22 06:33:43  flux
// First version
//
