// Copyright Erkki Seppl 2002
// $Id: addressipv4.cc,v 1.8 2003/01/26 19:40:39 flux Exp $
// $Author: flux $ 
// $Revision: 1.8 $

static char* gs_id __attribute__ ((unused)) ="$Id: addressipv4.cc,v 1.8 2003/01/26 19:40:39 flux Exp $"; /// ID for CVS/RCS

#include <cstdio>
#include <netdb.h>
#include <network/addressipv4.hh>
#include <netinet/in.h>

namespace Network {
  class AddressIPV4::InterfaceIPV4 : public Address::Interface {
  public:
    std::string asString(const sockaddr* a_address) const {
      const sockaddr_in* address = reinterpret_cast<const sockaddr_in*>(a_address);
      
      char buffer[4*4+1+5+1];
      
      in_addr_t addr = ntohl(address->sin_addr.s_addr);
      
      std::snprintf(buffer, sizeof(buffer), "%d.%d.%d.%d:%d",
                    (addr >> 24) & 0xff,
                    (addr >> 16) & 0xff,
                    (addr >>  8) & 0xff,
                    (addr >>  0) & 0xff,
                    ntohs(address->sin_port));
      
      return buffer;
    }
    socklen_t getSize(const sockaddr*) const {
      return sizeof(sockaddr_in);
    }
  } g_interface;

  class AddressIPV4::HelperIPV4 {
  public:
    HelperIPV4() {
      AddressIPV4::s_interfaces[AF_INET] = &g_interface;
    }
  } g_helper;

  AddressIPV4::AddressIPV4(std::string a_host, int a_port)
  {
    sockaddr_in address;
    hostent* host = gethostbyname2(a_host.c_str(), AF_INET);

    if (host && host->h_length) {
      address.sin_family = AF_INET;
      address.sin_port = htons(a_port);
      memcpy(&address.sin_addr, host->h_addr_list[0], 4);
    } else {
      error(ERROR_RESOLVE, "Cannot find such host");
    }
    
    set(&address);
  }

  AddressIPV4::AddressIPV4(in_addr_t a_address, int a_port)
  {
    sockaddr_in address;
    address.sin_family = AF_INET;
    address.sin_port = htons(a_port);
    address.sin_addr.s_addr = htonl(a_address);
    set(&address);
  }

  AddressIPV4::AddressIPV4(const Address& a_address)
  {
    int size;
    const sockaddr_in* address = reinterpret_cast<const sockaddr_in*>(a_address.get(&size));
    if (address->sin_family == AF_INET) {
      set(address);
    } else {
      error(ERROR_FAMILY, "Invalid family (not IPV4)");
    }
  }

  AddressIPV4::AddressIPV4(const AddressIPV4& a_address) :
    Address(a_address.get(0))
  {
    // nothing
  }

  int
  AddressIPV4::getProtocol()
  {
    return AF_INET;
  }

  in_addr_t
  AddressIPV4::getHost() const
  {
    const sockaddr_in* address = reinterpret_cast<const sockaddr_in*>(get(0));
    return ntohl(address->sin_addr.s_addr);
  }

  int
  AddressIPV4::getPort() const
  {
    const sockaddr_in* address = reinterpret_cast<const sockaddr_in*>(get(0));
    return ntohs(address->sin_port);
  }

  void
  AddressIPV4::setPort(int a_port)
  {
    sockaddr_in* address = reinterpret_cast<sockaddr_in*>(get(0));
    address->sin_port = htons(a_port);
  }

  void
  AddressIPV4::setHost(in_addr_t a_address)
  {
    sockaddr_in* address = reinterpret_cast<sockaddr_in*>(get(0));
    address->sin_addr.s_addr = htonl(a_address);
  }

  void
  AddressIPV4::setHost(std::string a_host)
  {
    sockaddr_in* address = reinterpret_cast<sockaddr_in*>(get(0));
    hostent* host = gethostbyname2(a_host.c_str(), AF_INET);

    if (host && host->h_length) {
      memcpy(&address->sin_addr, host->h_addr_list[0], 4);
    } else {
      error(ERROR_RESOLVE, "Cannot find such host");
    }
  }

  AddressIPV4
  AddressIPV4::withPort(int a_port) const
  {
    AddressIPV4 address = *this;
    address.setPort(a_port);
    return address;
  }
}


// changelog:
//
// $Log: addressipv4.cc,v $
// Revision 1.8  2003/01/26 19:40:39  flux
// Added some files (had to revert an old repository from backups)
//
// Revision 1.8  2002/07/02 19:01:39  flux
// Fixed typecasts
//
// Revision 1.7  2002/05/28 14:55:04  flux
// Updated copyright message
//
// Revision 1.6  2002/05/28 10:00:43  flux
// Removed ANY, BROADCAST and NONE - we're already including netinet/in.h
//
// Revision 1.5  2002/05/28 09:58:52  flux
// Added accessors and withPort
//
// Revision 1.4  2002/05/28 09:39:41  flux
// Added constructor for a raw address
// Added default addresses ANY, BROADCAST and NONE
//
// Revision 1.3  2002/05/22 08:33:29  flux
// Added some Socket-wrapping
//
// Revision 1.2  2002/05/22 07:03:04  flux
// Added error-handling.
// Added casting.
// Added type checking method.
//
// Revision 1.1  2002/05/22 06:33:43  flux
// First version
//
