// Copyright Erkki Seppl 2002
// $Id: addressipv4.hh,v 1.6 2002/05/28 14:55:04 flux Exp $
// $Author: flux $ 
// $Revision: 1.6 $


#ifndef ADDRESSIPV4_HH
#define ADDRESSIPV4_HH

#include <string>
#include <network/address.hh>
#include <netinet/in.h>

namespace Network {
  class AddressIPV4: public Address {
  public:
    AddressIPV4(const Address& a_address);
    AddressIPV4(const AddressIPV4& a_address);
    AddressIPV4(std::string a_host, int a_port);
    AddressIPV4(in_addr_t a_address, int a_port);

    in_addr_t 	getHost() const;
    int		getPort() const;

    void	setHost(std::string a_host);
    void	setHost(in_addr_t a_address);
    void	setPort(int a_port);
    AddressIPV4	withPort(int a_port) const;
    
    static int getProtocol();
    
  private:
    class InterfaceIPV4;
    friend class InterfaceIPV4;
    class HelperIPV4;
    friend class HelperIPV4; 
  };
}

#endif // ADDRESSIPV4_HH


// changelog:
//
// $Log: addressipv4.hh,v $
// Revision 1.6  2002/05/28 14:55:04  flux
// Updated copyright message
//
// Revision 1.5  2002/05/28 10:00:43  flux
// Removed ANY, BROADCAST and NONE - we're already including netinet/in.h
//
// Revision 1.4  2002/05/28 09:58:52  flux
// Added accessors and withPort
//
// Revision 1.3  2002/05/28 09:39:41  flux
// Added constructor for a raw address
// Added default addresses ANY, BROADCAST and NONE
//
// Revision 1.2  2002/05/22 07:03:04  flux
// Added error-handling.
// Added casting.
// Added type checking method.
//
// Revision 1.1  2002/05/22 06:33:43  flux
// First version
//
