// Copyright Erkki Seppl 2002
// $Id: addressunix.cc,v 1.1 2003/01/26 19:40:39 flux Exp $
// $Author: flux $ 
// $Revision: 1.1 $

static char* gs_id __attribute__ ((unused)) ="$Id: addressunix.cc,v 1.1 2003/01/26 19:40:39 flux Exp $"; /// ID for CVS/RCS

#include "addressunix.hh"
#include <unistd.h>

namespace Network {
  static const int MAX_UNIX_PATH = 108; // !!!!! UNIX_MAX_PATH
  
  class AddressUnix::InterfaceUnix : public Address::Interface {
  public:
    std::string asString(const sockaddr* a_address) const {
      sockaddr_un* address = (sockaddr_un*) a_address;

      return std::string(address->sun_path);
    }

    socklen_t getSize(const sockaddr*) const {
      return sizeof(sockaddr_un);
    }

  } g_interface;

  class AddressUnix::HelperUnix {
  public:
    HelperUnix() {
      AddressUnix::s_interfaces[AF_UNIX] = &g_interface;
    }
  } g_helper;

  AddressUnix::AddressUnix(const Address& a_address)
  {
    sockaddr_un* address = (sockaddr_un*) a_address.get(0);
    if (address->sun_family == AF_UNIX) {
      set(address);
    } else {
      error(ERROR_FAMILY, "Invalid family (not UNIX)");
    }
  }

  AddressUnix::AddressUnix()
  {
    sockaddr_un address;
    address.sun_family = AF_UNIX;
    address.sun_path[0] = 0;
    set(&address);
  }

  AddressUnix::AddressUnix(const AddressUnix& a_address) :
    Address(a_address.get(0))
  {
    // nothing
  }

  AddressUnix::AddressUnix(std::string a_path)
  {
    sockaddr_un address;
    address.sun_family = AF_UNIX;
    strncpy(address.sun_path, a_path.c_str(), MAX_UNIX_PATH), 
    set(&address);
  }

  std::string
  AddressUnix::getPath() const
  {
    return reinterpret_cast<const sockaddr_un*>(get(0))->sun_path;
  }

  void
  AddressUnix::setPath(std::string a_path)
  {
    strncpy(reinterpret_cast<sockaddr_un*>(get(0))->sun_path, a_path.c_str(), MAX_UNIX_PATH);
  }

  AddressUnix::~AddressUnix()
  {
    // nothing
  }

  void
  AddressUnix::unlink() const
  {
    ::unlink(reinterpret_cast<const sockaddr_un*>(get(0))->sun_path);
  }
}

// changelog:
//
// $Log: addressunix.cc,v $
// Revision 1.1  2003/01/26 19:40:39  flux
// Added some files (had to revert an old repository from backups)
//
// Revision 1.2  2002/07/02 19:54:47  flux
// Added unlink-method to AddressUnix
//
// Revision 1.1  2002/07/02 19:41:07  flux
// Added Unix-addresses
//
