// Copyright Erkki Seppl 2002
// $Id: socket.hh,v 1.9 2003/02/16 01:38:46 flux Exp $
// $Author: flux $ 
// $Revision: 1.9 $


#ifndef SOCKET_HH
#define SOCKET_HH

#include <memory>

#include <network/selectable.hh>

namespace Network {
  class Stream;
  
  class Address;
  
  class Socket: public Selectable {
  public:
    Socket();
    Socket(int a_fd);
    Socket(const Socket& a_other);
    Socket& operator=(const Socket& a_other); /// not there
    ~Socket();

    // assigns a new file descriptor
    void setFd(int a_fd);

    // returns the file descriptor
    int getFd() const;

    /// disowns the file descriptor (does not close it)
    void drop();

    /// reuse bind address
    void reuseAddress(bool a_reuse);
    
    /// binds the socket with given address
    bool bind(const Address& a_address, int* a_error = 0);

    /// starts listening for inbound connections
    bool listen(int a_backlog, int* a_error = 0);

    /// accepts a connection
    Socket accept(int* a_error = 0);

    /// accepts a connection
    Socket accept(Address& a_from, int* a_error = 0);
    
    /// established a connection
    bool connect(const Address& a_to, int* a_error = 0, bool a_nonblock = false);

    /// Tries to read given number of bytes (with proper retries)
    int readFull(void* a_buf, int a_count, int* a_error = 0);
    
    /// Tries to write given number of bytes (with proper retries)
    int writeFull(void* a_buf, int a_count, int* a_error = 0);
    
    /// Receives a packet
    int recv(void* a_buf, int a_count, int* a_error = 0);
    
    /// Receives a packet with sender address
    int recv(void* a_buf,
             int a_count,
             Address& a_from,
             int* a_error = 0);
    
    /// Sends an packet
    int send(const void* a_buf,
             int a_count,
             int* a_error = 0);

    /// Sends an UDP-packet to a given address
    int send(const void* a_buf,
             int a_count,
             const Address& a_to,
             int* a_error = 0);

    operator bool() const;

    Stream& stream();

    int markSelect(fd_set* a_readFds,
                   fd_set* a_writeFds,
                   fd_set* a_exceptFds);
    
    bool checkSelect(const fd_set* a_readFds,
                     const fd_set* a_writeFds,
                     const fd_set* a_exceptFds);
    
  private:
    int		m_fd;
    std::auto_ptr<Stream> m_stream;

    int 	copyFd() const;
  };
}

#endif // SOCKET_HH


// changelog:
//
// $Log: socket.hh,v $
// Revision 1.9  2003/02/16 01:38:46  flux
// Added non-blocking connect
//
// Revision 1.8  2002/05/28 18:30:01  flux
// Added support for SO_REUSEADDR
//
// Revision 1.7  2002/05/28 15:53:33  flux
// Added selecting support
//
// Revision 1.6  2002/05/28 15:17:54  flux
// Added writeFull
//
// Revision 1.5  2002/05/28 15:02:59  flux
// Added readFull
//
// Revision 1.4  2002/05/28 14:55:04  flux
// Updated copyright message
//
// Revision 1.3  2002/05/28 07:46:06  flux
// Added connect
//
// Revision 1.2  2002/05/22 08:33:29  flux
// Added some Socket-wrapping
//
// Revision 1.1  2002/05/22 06:33:43  flux
// First version
//
