// Copyright Erkki Seppl 2002
// $Id: stream.cc,v 1.4 2003/01/26 19:42:34 flux Exp $
// $Author: flux $ 
// $Revision: 1.4 $

static char* gs_id __attribute__ ((unused)) ="$Id: stream.cc,v 1.4 2003/01/26 19:42:34 flux Exp $"; /// ID for CVS/RCS

#include <network/stream.hh>
#include <unistd.h>
#include <sys/socket.h>

namespace Network {
  StreamBuf::StreamBuf(int a_fd) :
    m_fd(a_fd)
  {
    setp(m_writeBuffer, m_writeBuffer + sizeof(m_writeBuffer));
    setg(m_readBuffer, m_readBuffer, m_readBuffer);
  }

  StreamBuf::~StreamBuf()
  {
    sync();
  }

  int
  StreamBuf::overflow(int c)
  {
    sync();
    char ch = (char) c;
    if (write(m_fd, &ch, sizeof(ch) <= 0)) {
      return EOF;
    } else {
      return 0;
    }
  }
  
  int
  StreamBuf::underflow()
  {
    if (sync() == EOF) {
      return EOF;
    }
    int n = read(m_fd, m_readBuffer, sizeof(m_readBuffer));
    if (n <= 0) {
      return EOF;
    } else {
      setg(m_readBuffer, m_readBuffer, m_readBuffer + n);
      gbump(1);
      return m_readBuffer[0];
    }
  }
  
  int
  StreamBuf::uflow()
  {
    return underflow();
  }
  
  int
  StreamBuf::sync()
  {
    char* putBegin = m_writeBuffer;
    while (putBegin < pptr()) {
      int n = write(m_fd, putBegin, pptr() - putBegin);
      if (n <= 0) {
        return EOF;
      } else {
        putBegin += n;
      }
    }
    setp(m_writeBuffer, m_writeBuffer + sizeof(m_writeBuffer));

    return 0;
  }

  int
  StreamBuf::getFd() const
  {
    return m_fd;
  }

  Stream::Stream(int a_fd) :
    std::iostream(new StreamBuf(a_fd))
  {
    // nothing
  }

  Stream::~Stream()
  {
    delete rdbuf();
  }

  void
  Stream::disconnect()
  {
    shutdown(static_cast<StreamBuf*>(rdbuf())->getFd(), SHUT_RDWR);
  }
}

// changelog:
//
// $Log: stream.cc,v $
// Revision 1.4  2003/01/26 19:42:34  flux
// Fixed c++ standard version bugs older gcc hadn't noticed
//
// Revision 1.3  2002/05/28 14:55:04  flux
// Updated copyright message
//
// Revision 1.2  2002/05/28 07:47:16  flux
// Added actual stream code.
// Added Stream::disconnect
//
// Revision 1.1  2002/05/22 08:33:29  flux
// Added some Socket-wrapping
//
