// Copyright Erkki Seppl 2001-2002
// $Id: occurrence.cc,v 1.2 2002/05/17 18:58:33 flux Exp $
// $Author: flux $ 
// $Revision: 1.2 $

static char* gs_id __attribute__ ((unused)) ="$Id: occurrence.cc,v 1.2 2002/05/17 18:58:33 flux Exp $"; /// ID for CVS/RCS

#include "occurrence.hh"

Occurrence::Occurrence()
{
  // nothing
}

Occurrence::~Occurrence()
{
  // nothing
}

OneShotOccurrence::OneShotOccurrence(time_t a_time) :
  m_time(a_time)
{
  // nothing
}

time_t
OneShotOccurrence::nextAfter(time_t a_time) const
{
  if (a_time >= m_time) {
    return 0;
  } else {
    return m_time;
  }
}

time_t
OneShotOccurrence::lastBefore(time_t a_time) const
{
  if (a_time > m_time) {
    return m_time;
  } else {
    return 0;
  }
}

void
OneShotOccurrence::print(std::ostream& stream) const
{
  stream << "at " << formatTime(m_time, "%D %H:%M");
}

RangeOccurrence::RangeOccurrence(time_t a_begin,
                                 time_t a_end,
                                 Occurrence* a_occurrence) :
  m_begin(a_begin),
  m_end(a_end),
  m_occurrence(a_occurrence)
{
  // nothing
}

time_t
RangeOccurrence::nextAfter(time_t a_time) const
{
  time_t t = m_occurrence->nextAfter(a_time);
  if (t >= m_begin && t < m_end) {
    return t;
  } else {
    return 0;
  }
}

time_t
RangeOccurrence::lastBefore(time_t a_time) const
{
  time_t t = m_occurrence->lastBefore(a_time);
  if (t >= m_begin && t < m_end) {
    return t;
  } else {
    return 0;
  }
}

void
RangeOccurrence::print(std::ostream& stream) const
{
  stream << "between "
         << formatTime(m_begin, "%D")
         << " and "
         << formatTime(m_end, "%D") << ", ";
  m_occurrence->print(stream);  
}

CycleOccurrence::CycleOccurrence(time_t a_zero,
                                 int a_cycle) :
  m_zero(a_zero),
  m_cycle(a_cycle)
{
  // nothing
}

time_t
CycleOccurrence::nextAfter(time_t a_time) const
{
  if (a_time <= m_zero) {
    return m_zero;
  } else {
    return a_time - (a_time - m_zero) % m_cycle + m_cycle;
  }
}

time_t
CycleOccurrence::lastBefore(time_t a_time) const
{
  if (a_time <= m_zero) {
    return 0;
  } else {
    return a_time - (a_time - m_zero) % m_cycle;
  }
}

void
CycleOccurrence::print(std::ostream& stream) const
{
  static const struct {
    char* format;
    int length;
    char* singular;
    char* plural;
  } shortcuts[] = {
    { "on %m/%d at %H:%M every ", 365 * 24 * 60 * 60, "year", "years" },
    { "on %d. day at %H:%M every ", 30 * 24 * 60 * 60, "month", "months" },
    { "on %a at %H:%M every ", 7 * 24 * 60 * 60, "week", "weeks" },
    { "%H:%M every ", 24 * 60 * 60, "day", "days" },
    { "at %M minutes past every ", 60 * 60, "hour", "hours" },
    { "every ", 60, "minute", "minutes" },
    { "every ", 1, "second", "seconds" }
  };
  for (int c = 0; /* last shortcut always matches */ ; ++c) {
    if (m_cycle % shortcuts[c].length == 0) {
      stream << formatTime(m_zero, shortcuts[c].format);
      if (m_cycle / shortcuts[c].length == 1) {
        stream << shortcuts[c].singular;
      } else {
        stream << m_cycle / shortcuts[c].length << " " << shortcuts[c].plural;
      }
      break;
    }
  }
}

NeverOccurrence::NeverOccurrence()
{
  // nothing
}

time_t
NeverOccurrence::nextAfter(time_t) const
{
  return 0;
}

time_t
NeverOccurrence::lastBefore(time_t) const
{
  return 0;
}

void
NeverOccurrence::print(std::ostream& stream) const
{
  stream << "never";
}

bool
overlaps(time_t time1, int minutes1,
         time_t time2, int minutes2)
{
  if (time1 < time2) {
    return time2 < (time1 + minutes1 * 60);
  } else {
    return time1 < (time2 + minutes2 * 60);
  }
}

std::ostream&
operator<<(std::ostream& a_stream, const Occurrence& a_occurrence)
{
  a_occurrence.print(a_stream);
  return a_stream;
}

std::string
formatTime(time_t a_time, std::string a_format)
{
  char buffer[1024];
  tm* t = localtime(&a_time);
  strftime(buffer, sizeof(buffer), a_format.c_str(), t);
  return buffer;
}


// changelog:
//
// $Log: occurrence.cc,v $
// Revision 1.2  2002/05/17 18:58:33  flux
// Updated copyright message
//
// Revision 1.1  2001/11/04 13:30:58  flux
// First version
//
