// Copyright Erkki Seppl 2001-2002
// $Id: occurrence.hh,v 1.2 2002/05/17 18:58:33 flux Exp $
// $Author: flux $ 
// $Revision: 1.2 $


#ifndef OCCURRENCE_HH
#define OCCURRENCE_HH

#include <memory>
#include <iostream>
#include <string>

#include <time.h>

class Occurrence {
public:
  Occurrence();
  virtual ~Occurrence();

  /** returns the time when the next show would be shown, or 0 if no such show
   * \param a_time unix time (GMT), inclusive */
  virtual time_t nextAfter(time_t a_time) const = 0;
  /** returns the time when the last show ended, or 0 if no such show
   * \param a_time unix time (GMT), exclusive */
  virtual time_t lastBefore(time_t a_time) const = 0;

  virtual void print(std::ostream& stream) const = 0;
};

class OneShotOccurrence: public Occurrence {
public:
  OneShotOccurrence(time_t time);
  time_t nextAfter(time_t a_time) const;
  time_t lastBefore(time_t a_time) const;
  void print(std::ostream& stream) const;
private:
  time_t m_time;
};

class RangeOccurrence: public Occurrence {
public:
  RangeOccurrence(time_t a_begin,
                  time_t a_end,
                  Occurrence* a_occurrence);

  time_t nextAfter(time_t a_time) const;
  time_t lastBefore(time_t a_time) const;
  void print(std::ostream& stream) const;

private:
  time_t m_begin;
  time_t m_end;
  std::auto_ptr<Occurrence> m_occurrence;
};

class CycleOccurrence: public Occurrence {
public:
  CycleOccurrence(time_t a_zero,
                  int a_cycle);
  time_t nextAfter(time_t a_time) const;
  time_t lastBefore(time_t a_time) const;
  void print(std::ostream& stream) const;

private:
  time_t m_zero;
  int    m_cycle;
};

class NeverOccurrence: public Occurrence {
public:
  NeverOccurrence();
  time_t nextAfter(time_t a_time) const;
  time_t lastBefore(time_t a_time) const;
  void print(std::ostream& stream) const;
};

// utility functions
bool overlaps(time_t time1, int minutes1,
              time_t time2, int minutes2);

std::ostream& operator<<(std::ostream& a_stream, const Occurrence& a_occurrence);

std::string formatTime(time_t time, std::string format);

#endif // OCCURRENCE_HH


// changelog:
//
// $Log: occurrence.hh,v $
// Revision 1.2  2002/05/17 18:58:33  flux
// Updated copyright message
//
// Revision 1.1  2001/11/04 13:30:58  flux
// First version
//
