// Copyright Erkki Seppl 2002
// $Id: server.cc,v 1.2 2003/02/16 02:02:19 flux Exp $
// $Author: flux $ 
// $Revision: 1.2 $

static char* gs_id __attribute__ ((unused)) ="$Id: server.cc,v 1.2 2003/02/16 02:02:19 flux Exp $"; /// ID for CVS/RCS

#include <network/addressunix.hh>
#include <network/socketunix.hh>
#include <network/stream.hh>
#include <iostream>

#include <unistd.h>
#include <sys/time.h>

#include "server.hh"

using namespace Network;

Server::Server(std::string a_socketfile,
               std::ostream& a_log) :
  m_log(a_log),
  m_error(false)
{
  AddressUnix address(a_socketfile);
  SocketUnix sock(SocketUnix::STREAM);
  if (!sock.connect(address, 0, true)) {
    address.unlink();
    if (!sock.bind(address)) {
      m_log << "Failed to bind\n";
      m_error = true;
    } else {
      if (!sock.listen(0)) {
        m_log << "Failed to listen\n";
        m_error = true;
      } else {
        m_socket = sock;
      }
    }
  } else {
    m_log << "Server already running\n";
    m_error = true;
  }
}

Server::~Server()
{
  // nothing
}

Server::operator bool() const
{
  return !m_error;
}

int
Server::sleepAndServe(int a_seconds)
{
  fd_set readFds;
  FD_ZERO(&readFds);

  time_t end = time(0) + a_seconds;
  
  int n = m_socket.markSelect(&readFds, 0, 0);

  time_t now;

  // !!!! notice time warps
  while ((now = time(0)) < end) {
    timeval tv = { tv_sec : end - now, tv_usec : 0 };
    int rc = select(n, &readFds, 0, 0, &tv);
    if (rc > 0) {
      if (m_socket.checkSelect(&readFds, 0, 0)) {
        m_clients.push_back(PClient(new Client(m_socket.accept())));
      } else {
        for (ClientList::iterator it = m_clients.begin();
             it != m_clients.end();
             ++it) {
          (*it)->checkSelect(&readFds, 0, 0);
        }
      }
    } else if (rc < 0) {
      break; // time might not have elapsed
    }
  }

  return now - end;
}

//////////////////////////////////////////////////////////////////////
// Client
//////////////////////////////////////////////////////////////////////

Server::Client::Client(Network::Socket a_socket) :
  m_socket(a_socket)
{
  // nothing
}

Server::Client::~Client()
{
  // nothing
}

int
Server::Client::markSelect(fd_set* a_readFds,
                           fd_set* a_writeFds,
                           fd_set* a_exceptFds)
{
  return m_socket.markSelect(a_readFds, a_writeFds, a_exceptFds);
}

bool
Server::Client::checkSelect(const fd_set* a_readFds,
                            const fd_set* a_writeFds,
                            const fd_set* a_exceptFds)
{
  if (m_socket.checkSelect(a_readFds, a_writeFds, a_exceptFds)) {
    std::string cmd;
    std::getline(m_socket.stream(), cmd);
    std::cout << "Got command: " << cmd << "\n";
    return true;
  } else {
    return false;
  }
}

bool
Server::Client::disconnected()
{
  return !m_socket.stream();
}

// changelog:
//
// $Log: server.cc,v $
// Revision 1.2  2003/02/16 02:02:19  flux
// Several enhancements (now has the concept of client)
//
// Revision 1.1  2003/01/26 19:46:30  flux
// Committing most recent files to old repository backup
//
