// Copyright Erkki Seppl 2002
// $Id: server.hh,v 1.2 2003/02/16 02:02:19 flux Exp $
// $Author: flux $ 
// $Revision: 1.2 $


#ifndef SERVER_HH
#define SERVER_HH

#include <network/socket.hh>
#include <list>
#include <iostream>
#include <boost/shared_ptr.hpp>

#include "disconnectable.hh"

class Server {
public:
  Server(std::string a_socketfile,
         std::ostream& a_log);
  virtual ~Server();

  int sleepAndServe(int a_seconds);
  
  operator bool() const;

private:
  Server(const Server& a_other); /// not there
  Server& operator=(const Server& a_other); /// not there

  Network::Socket	m_socket;
  std::ostream&		m_log;
  bool			m_error;

  class Client: public Network::Selectable,
                public Disconnectable {
  public:
    Client(Network::Socket m_socket);

    ~Client();
    
    int markSelect(fd_set* a_readFds,
                   fd_set* a_writeFds,
                   fd_set* a_exceptFds);
    bool checkSelect(const fd_set* a_readFds,
                     const fd_set* a_writeFds,
                     const fd_set* a_exceptFds);

    bool disconnected();

  private:
    Network::Socket	m_socket;
  };

  typedef boost::shared_ptr<Client> PClient;
  typedef std::list<PClient> ClientList;
  ClientList		m_clients;
};

#endif // SERVER_HH


// changelog:
//
// $Log: server.hh,v $
// Revision 1.2  2003/02/16 02:02:19  flux
// Several enhancements (now has the concept of client)
//
// Revision 1.1  2003/01/26 19:46:30  flux
// Committing most recent files to old repository backup
//
