// Copyright Erkki Seppl 2001-2002
// $Id: show.cc,v 1.6 2003/01/26 19:50:06 flux Exp $
// $Author: flux $ 
// $Revision: 1.6 $

static char* gs_id __attribute__ ((unused)) ="$Id: show.cc,v 1.6 2003/01/26 19:50:06 flux Exp $"; /// ID for CVS/RCS

#include "show.hh"
#include "occurrence.hh"

Show::Show(std::string 	a_name,
           int 		a_minutes,
           int		a_priority,
           Device* 	a_device,
           std::string	a_channel,
           Occurrence*	a_occurrence,
	   int		a_extraMinutes,
           const Arguments& a_arguments) :
  m_name(a_name),
  m_minutes(a_minutes),
  m_priority(a_priority),
  m_device(a_device),
  m_channel(a_channel),
  m_occurrence(a_occurrence),
  m_lastRecording(0),
  m_extraMinutes(a_extraMinutes),
  m_arguments(a_arguments)
{
  if (!m_occurrence.get()) {
    m_occurrence.reset(new NeverOccurrence());
  }
}

Show::~Show()
{
  // nothing
}

Arguments
Show::getArguments() const
{
  return m_arguments;
}

Occurrence*
Show::getOccurrence()
{
  return m_occurrence.get();
}

void
Show::setOccurrence(Occurrence* a_occurrence)
{
  m_occurrence.reset(a_occurrence);
}

void
Show::setLastRecording(time_t a_time)
{
  m_lastRecording = a_time;
}

time_t
Show::getLastRecording() const
{
  return m_lastRecording;
}

Device*
Show::getDevice()
{
  return m_device;
}

time_t
Show::nextBegins(time_t a_time) const
{
  time_t next = m_occurrence->nextAfter(a_time);
  return next;
}

time_t
Show::lastBegun(time_t a_time) const
{
  time_t t = m_occurrence->lastBefore(a_time);
  return t;
}

time_t
Show::lastEnded(time_t a_time) const
{
  time_t t = m_occurrence->lastBefore(a_time);
  if (t) {
    t += m_minutes * 60;
  }
  return t;
}

int
Show::getMinutes() const
{
  return m_minutes;
}

int
Show::getPriority() const
{
  return m_priority;
}

int
Show::getExtra() const
{
  return m_extraMinutes;
}

std::string
Show::getName() const
{
  return m_name; 
}

std::string
Show::getChannel() const
{
  return m_channel; 
}

std::ostream&
operator<<(std::ostream& stream, const Show& show)
{
  stream << show.m_name
         << " (" << show.m_minutes;
  if (show.m_extraMinutes) {
    stream << "+" << show.m_extraMinutes;
  }
  stream <<" min) on " << show.m_channel;
  return stream;
}


// changelog:
//
// $Log: show.cc,v $
// Revision 1.6  2003/01/26 19:50:06  flux
// Committing most recent files to old repository backup
//
// Revision 1.5  2002/05/17 18:58:33  flux
// Updated copyright message
//
// Revision 1.4  2002/05/14 12:30:41  flux
// Show extra minutes
//
// Revision 1.3  2002/04/21 20:24:44  flux
// Preliminary support for "extra minutes".
// Fixed daylight saving time-related bug
//
// Revision 1.2  2001/11/15 14:33:13  flux
// Added support for remembering when the show was last time recorded
//
// Revision 1.1  2001/11/04 13:30:58  flux
// First version
//
