// Copyright Erkki Seppl 2001-2002
// $Id: show.hh,v 1.5 2003/01/26 19:50:06 flux Exp $
// $Author: flux $ 
// $Revision: 1.5 $


#ifndef SHOW_HH
#define SHOW_HH

#include <string>
#include <memory>
#include <iostream>

#include "arguments.hh"

class Device;
class Occurrence;

class Show {
public:
  Show(std::string 	a_name,
       int 		a_minutes,
       int		a_priority,
       Device* 		a_device,
       std::string	a_channel,
       Occurrence*	a_occurrence,
       int		a_extraMinutes,
       const Arguments&	a_arguments);

  Occurrence* getOccurrence();
  void setOccurrence(Occurrence* a_occurrence);

  time_t nextBegins(time_t a_time) const;
  time_t lastBegun(time_t a_time) const;
  time_t lastEnded(time_t a_time) const;

  void setLastRecording(time_t a_time);
  time_t getLastRecording() const;
  
  int getMinutes() const;
  int getExtra() const;
  int getPriority() const;
  std::string getName() const;
  Device* getDevice();
  std::string getChannel() const;
  Arguments getArguments() const;

  ~Show();

  friend std::ostream& operator<<(std::ostream& stream, const Show& show);
  
private:
  std::string 	m_name;         /// the name of the show
  int		m_minutes;      /// in minutes
  int		m_priority;     /// the lower the better
  Device*	m_device;       /// which device to use to record this show
  std::string	m_channel;      /// which channel to tune into
  std::auto_ptr<Occurrence> m_occurrence; /// when this show is shown
  time_t        m_lastRecording; /// last time this show was recorded
  int		m_extraMinutes; /// Store this many extra minutes if possible
  const Arguments m_arguments;  /// Extra arguments for recording
};

std::ostream& operator<<(std::ostream& stream, const Show& show);

#endif // SHOW_HH


// changelog:
//
// $Log: show.hh,v $
// Revision 1.5  2003/01/26 19:50:06  flux
// Committing most recent files to old repository backup
//
// Revision 1.4  2002/05/17 18:58:33  flux
// Updated copyright message
//
// Revision 1.3  2002/04/21 20:24:44  flux
// Preliminary support for "extra minutes".
// Fixed daylight saving time-related bug
//
// Revision 1.2  2001/11/15 14:33:13  flux
// Added support for remembering when the show was last time recorded
//
// Revision 1.1  2001/11/04 13:30:58  flux
// First version
//
